Imports System.Threading

Public Class TreeClass

    Private DatabaseCount As Integer
    Private CurrentCount As Integer
    Private _dbHelper As New DatabaseHelper

    ' This event is raised back to the calling 
    ' thread to set properties on the 
    ' progress bar
    Public Event UpdateProgress _
        (ByVal CurrentCount As Integer, _
         ByVal DatabaseCount As Integer)

    ' This event is raised back to the secondary
    ' thread created to invoke the update on the 
    ' TreeView control
    Public Event AddNode _
        (ByVal CurrentNode As TreeNode, _
         ByVal ParentNode As TreeNode, _
         ByVal ChildNode As TreeNode)

    Public Event ShowComplete(ByVal TotalCount As Integer)

    ' The only public method in the class.  This returns the top level 
    ' type nodes for Tables, Stored Procedures and Views, and 
    ' calls the subsequent private procedure which iterates thru 
    ' the database objects and returns a node for each
    ' of the top level types
    Public Sub LoadDatabaseTree()

        ' The _dbHelper class contains various 
        ' procedures that return datasets for 
        ' system objects of the current 
        ' database being loaded in the TreeView
        Dim ds As DataSet
        ds = _dbHelper.GetDatabases _
            (ConnectionInfo.Cn)

        DatabaseCount = _
            ds.Tables(0).Rows.Count

        For Each dr As DataRow _
            In ds.Tables(0).Rows

            ' Create a new TreeNode object 
            ' which is the database
            Dim n As New TreeNode
            With n
                ' Set properties on the TreeNode.  In this case, we are setting 
                ' properties like ImageIndex.  This does not make this class 
                ' completely generic, but I figured while we were here we could
                ' do it.  You might want to move the properties to the main form,
                ' since that is where the ImageList exists.  The only difference
                ' ends up being how generic this class needs to be
                .Text = dr("name") & ""
                .Tag = "Database"
                .ImageIndex = 2
                .SelectedImageIndex = 2
                ' Raise the event back to the calling thread to add the node to the 
                ' existing TreeView in the form
                RaiseEvent AddNode(Nothing, Nothing, n)
                ' Load the current database with it's tables
                Me.AddTableNodes(n, "", "Tables")
                ' Load the Stored Procedures for the current database
                Me.AddTableNodes(n, dr("name"), _
                        "Stored Procedures")
                ' Load the Views for the current database
                Me.AddTableNodes(n, dr("name"), _
                        "Views")

                ' Update the counter in this class, which is passed
                ' to the event on the calling thread
                CurrentCount += 1
                ' Raise an event back to the calling thread to update then
                ' progress bar count
                RaiseEvent UpdateProgress _
                        (CurrentCount, DatabaseCount)

            End With
        Next
        ' Raise an event to the calling thread that we are complete
        ' with loading all of this server's objects
        RaiseEvent ShowComplete(CurrentCount)
    End Sub

    Private Sub AddTableNodes(ByVal CurrentNode As TreeNode, _
                    ByVal DatabaseName As String, _
                    ByVal WhichOne As String)

        Dim dsTables As New DataSet

        ' Determine what we are supposed to be getting based on the 
        ' WhichOne variable, which can be Tables, Views or 
        ' Stored Procedures, and call the correct method in the _dbHelper
        ' class to get the system objects we need to get the nodes for
        Select Case WhichOne.ToLower()
            Case "tables"
                dsTables = _dbHelper.GetTables _
                        (CurrentNode.Text, ConnectionInfo.Cn)
            Case "views"
                dsTables = _dbHelper.GetViews _
                        (DatabaseName, ConnectionInfo.Cn)
            Case "stored procedures"
                dsTables = _dbHelper.GetStoredProcedures _
                        (DatabaseName, ConnectionInfo.Cn)
        End Select

        ' Add the parent node and raise an event
        ' to the calling thread to update the TreeNode
        Dim nParent As New TreeNode(WhichOne)
        With nParent
            .ImageIndex = 6
            .SelectedImageIndex = 6
        End With
        ' Raise the event in this class, which is being handled on the calling
        ' thread on the form.  The method for this event on MainForm is not
        ' executing on the main thread of the UI, it is on the thread you created
        ' This node ends up being the top level node for the object type from
        ' the database, Tables, Views or Stored Procedures
        RaiseEvent AddNode(CurrentNode, nParent, Nothing)

        ' Loop thru the dataset and create the node to return back to the caller
        For Each dr As DataRow In dsTables.Tables(0).Rows
            Dim n As New TreeNode
            With n
                .Text = dr(0) & ""
                .Tag = WhichOne
                Select Case WhichOne.ToLower
                    Case "tables"
                        .ImageIndex = 3
                        .SelectedImageIndex = 3
                    Case "stored procedures"
                        .ImageIndex = 4
                        .SelectedImageIndex = 4
                    Case "views"
                        .ImageIndex = 5
                        .SelectedImageIndex = 5
                End Select
                ' Raise an event to the calling thread to update the TreeNode.  The
                ' data for this node is an actual Table, View or Stored Procedure
                ' object that is a child of a child for the database we are currently 
                ' iterating objects for
                RaiseEvent AddNode(CurrentNode, nParent, n)
            End With
        Next
    End Sub

End Class
